/*
    \brief Deklarationen für die Funktionen des Systemlogs.

    \author Bernd Ottmann
    \date   21.02.2003
*/
#pragma once
#ifndef DATAFOX_SYSTEM_RECORDS_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define DATAFOX_SYSTEM_RECORDS_H
#endif

#include "df_defines.h"

#if (DF_API_VERSION >= 0x04010700)

/*!
    \brief Logdatensatz aus Gerät lesen.

    Es wird versucht einen Logdatensatz aus dem Gerät zu lesen. Nach erfolgreicher Vararbeitung muss dieser mit
    DFCSystemQuitRecord() quittiert werden, damit weitere Datensätze abgerufen werden können. Wird ein Datensatz nicht
    mit DFCSystemQuitRecord() quittiert wird er als ein Wiederholt gelieferter Datensatz gemeldet.

    \ingroup system_records

    \export{_DFCSystemReadRecord@20, DFCSystemReadRecord}

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] buffer Datenpuffer.
    \param[in,out] bufferSize Als \c in die reservierte Größe von \a buffer, als \c out die übergebene Datensatzlänge.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval -1 Fehler bei Ausführung.
    \retval 0 Keine Datensätze vorhanden.
    \retval 1 Datensatz gelesen.

    \since 04.01.07 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCSystemReadRecord( int connectionId, int deviceAddress, unsigned char *buffer, int *bufferSize, int *errorNumber );

/*!
    \brief Logdatensatz im Gerät quittieren.

    Es wird versucht einen zuvor mit DFCSystemReadRecord() gelesenen Logatensatz zu quittieren. Nach erfolgreicher Quittierung
    kann mit einem folgenden Aufruf von DFCSystemReadRecord() der ggf. nächste Datensatz gelesen werden.

    \ingroup system_records

    \export{_DFCSystemQuitRecord@12, DFCSystemQuitRecord}

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval -1 Fehler bei Ausführung.
    \retval 0 Keine Quittung benötigt.
    \retval 1 Datensatz quittiert.

    \since 04.01.07 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCSystemQuitRecord( int connectionId, int deviceAddress, int *errorNumber );

/*!
    \brief Logdatensätze wiederherstellen.

    Durch ein Zurücksetzen der Datensatzzeiger auf den letzten auslesbaren Logdatensatz wird ein Wiederauslesen über
    DFCSystemReadRecord() möglich. Durch Verwendung eines Ringspeichers in den Geräten werden ggf. ältere Datensätze durch
    aktuellere überschrieben.

    \ingroup system_records

    \export{_DFCSystemRestoreRecords@12, DFCSystemRestoreRecords}

    \warning Durch ein Wiederherstellen der Datensätze kann eine ggf. sehr große Datensatzmenge entstehen.

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.07 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCSystemRestoreRecords( int connectionId, int deviceAddress, int *errorNumber );

#endif

#endif
