#include "api/df_system_records.h"
#include "datafox_library_iv.h"

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSystemReadRecord(int connectionId, int deviceAddress, unsigned char *buffer, int *bufferSize, int *errorNumber)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || buffer == nullptr || bufferSize == nullptr || errorNumber == nullptr)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        
        channelBridge->logEnd("\t\t\t%i\t(%i, %i)", *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    int res = channelBridge->systemReadRecord(deviceAddress, buffer, (std::size_t &)*bufferSize, (int32_t &)*errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (res == -1)
    {
        channelBridge->setLastErrorNumber(deviceAddress, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));
        
        channelBridge->logAppend("\t\t");

    }
    else if (res == 0)
    {
        channelBridge->logAppend("\t\t");
    }
    else
    {
        // Daten fuer Log in Puffer uebergeben
        unsigned char ucTmp[MAX_MB_PAKET_SYSTEM_DATA * 3 + 1];
        int i;
        for (i=0; i < *bufferSize; i++)
        {
            sprintf((char*)&ucTmp[i * 3], "%02X ", (unsigned char)buffer[i]);
        }
        ucTmp[i * 3] = 0;

        channelBridge->logAppend("\t%s\t%i", ucTmp, *bufferSize);
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, res, channelBridge->getLastErrorNumber(deviceAddress));

    return res;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSystemQuitRecord( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false || errorNumber == nullptr)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        
        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    int res = channelBridge->systemQuitRecord(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (res == -1)
    {
        channelBridge->setLastErrorNumber(deviceAddress, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, res, channelBridge->getLastErrorNumber(deviceAddress));

    return res;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSystemRestoreRecords( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false || errorNumber == nullptr)
    {
        channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        
        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->systemRestoreRecords(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (iRes == 0)
    {
        channelBridge->setLastErrorNumber(deviceAddress, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}
