#pragma once
#ifndef HARDWARE_INFO_STRUCTS_H
#define HARDWARE_INFO_STRUCTS_H

#pragma pack(1)
///////////////////////////////////////
// Struct: StructHardwareInfo

/*!
    \brief Diese Struktur enthaelt die Informationen zum Geraet.
*/
struct StructHardwareInfo
{
    unsigned char version;
    unsigned char typeDevice;
    unsigned char typeBoard;
    unsigned char componentRS485_0;
    unsigned char typeTCPIP;
    unsigned char typeModemMC;
    unsigned char componentRS485_1;
    unsigned char typeTransponder;
    unsigned char typeDisplay;
    unsigned char typeKeypad;
    unsigned char typeMMC;
    unsigned char typeGPS;
    unsigned char componentUSB;
    unsigned char typeRadio;
    unsigned char typeDCF77;
    unsigned int componentInAnalog;
    unsigned int componentInDigital;
    unsigned int componentOutDigital;
    unsigned char typeFingerprint;
    unsigned char typeExternalReader;
    unsigned char optionIOModule;
    unsigned char optionTBAdaption;
    // Zutrittskontrolle 
#define HIAF_HAS_ZK       0x01 // Geraet nutzt Zutrittskontrolle

#define HIAF_HAS_OPTIONS2 0x02
    unsigned char administrationFlags;
    unsigned char optionComserver;
    unsigned char optionTPWrite;
    unsigned char optionHttp;
    unsigned char optionLanguage;
    unsigned char optionBiokeyUpdate;
#define HIAF_HAS_OPTIONS3 0x04
    unsigned short optionsSetup;
#define HIAF_HAS_OPTIONS4  0x08
    unsigned char typeTouch;
    unsigned char typeSensor;
    unsigned char componentBattery;
    unsigned char componentCharge;
#define HIAF_HAS_OPTIONS5 0x10
    unsigned char optionsPayable;

};

/*!
    \brief Headerstruktur für die gesamte Hardwareinfo.
*/ 
struct HARDWARECONFIGNODE
{
    unsigned char version;                  //!< Versionsnummer der Konfigurationsstrukturen
    unsigned char flags;                    //!< Flag-Feld fuer Konfigurationsuebergreifende Optionen.
                                            //!< Bits: Unused
    unsigned short device;                  //!< Geraetetypnummer. // -stimmt nicht mit dem Studio überein da einmal ein Zahlendreher gewesen ist
    unsigned int serial;                    //!< Seriennummer des Geraetes. // nicht genutzt vom HIP bzw. nur als fortlaufende Platinennummer
    unsigned short count;                   //!< Anzahl der folgenden Konfigurationsstrukturen.
                                            //!< Die Platznummer 0 ist fuer die Hauptplatine reserviert.
    unsigned short signature;               //!< Signatur der Hardware-Configuration.
    unsigned int signatureSize;             //!< Anzahl Byte Signaturrelevanter Daten.
    unsigned int ptrHardwareConfigArray;    //!< Offset auf das folgende Pointerarray der Offsets auf die einzelnen Folgestrukturen vom Typ HARDWARECONFIG.
                                            //!< Diese HARDWARECONFIG Strukturen können über eine HARDWARECONFIGSUBNODE Struktur über wiederum eigene HARDWARECONFIG's verfügen.
}; // 20 Byte

/*!
    \brief Struktur der Hardware-Konfiguration eines Moduls.
*/
struct HARDWARECONFIG
{
    unsigned short number;                     //!< Platznummer dieser Konfiguration.
    unsigned int id;                           //!< Typnummer des zugewiesenen Moduls.
    unsigned char revision;                    //!< Revision des Moduls. Bespiel 11 bedeutet Version 1.1  
    unsigned char index;                       //!< Bestuecklistenindex des Moduls. Beispiel: 1 = a, 2 = b, 3 = c  
    char article[11];                          //!< Interne Artikelnummer des Moduls.
    unsigned char type;                        //!< Type des Eintrags. Submodule oder Adapter etc. 0 = Mainboard, 1 = Modul, 2 = Modul mit Plätzen, 3 = Submodul, 4 = Erweiterungsplatine
                                               //!< Unterschied Modul und Submodul. Submodul kann auf ein Modul mit Plätzen bestückt werden
    char name[25];                             //!< Bezeichnung des Moduls. Zur Anzeige nutzbar.
    unsigned int ptrHardwareConfigSubNode;     //!< Offset auf die Folgestruktur vom Typ HARDWARECONFIGSUBNODE.
    unsigned char jumperCount;                 //!< Anzahl der zugeordneten Jumperstrukturen.
    unsigned int ptrHardwareConfigJumperArray; //!< Offset auf das folgende Pointerarray der Offsets auf die einzelnen Strukturen vom Typ HARDWAREJUMPER.
}; // 54 Byte

/*!
    \brief Folgestruktur für Jumperdefinitionen der Platzteile.
*/
struct HARDWARECONFIGJUMPER
{
    unsigned char number;       //!< Nummer dieses Jumpers.
    unsigned char flags;        //!< Zusätliche Angaben.
                                //!< Bits: Unused
    unsigned char count;        //!< Anzahl der Pins.
    unsigned char start;        //!< Startpin einer Brücke.
    unsigned char end;          //!< Endpin einer Brücke.
    char name[25];              //!< Bezeichnung des Moduls. Zur Anzeige nutzbar.
}; //30 Byte

/*!
    \brief Folgestruktur der Module mit eigenen Plaetzen.
*/
struct HARDWARECONFIGSUBNODE
{
    unsigned char flags;                    //!< Flag-Feld fuer Konfigurationsuebergreifende Optionen.
                                            //!< Bits: Unused
    unsigned short count;                   //!< Anzahl der folgenden Konfigurationsstrukturen.
    unsigned int ptrHardwareConfigArray;    //!< Offset auf das folgende Pointerarray der Offsets auf die einzelnen Folgestrukturen vom Typ HARDWARECONFIG.
}; // 7 Byte

#pragma pack()

#endif
